@@ -31,10 +31,11 @@
 #include <gudev/gudev.h>
 
 #include "up-backend.h"
-#include "up-backend-linux-private.h"
 #include "up-daemon.h"
 #include "up-device.h"
 
+#include "up-enumerator-udev.h"
+
 #include "up-device-supply.h"
 #include "up-device-wup.h"
 #include "up-device-hid.h"
@@ -58,12 +59,14 @@ struct UpBackendPrivate
 	UpDaemon		*daemon;
 	UpDeviceList		*device_list;
 	GUdevClient		*gudev_client;
-	UpDeviceList		*managed_devices;
+	UpInput			*lid_device;
 	UpConfig		*config;
 	GDBusProxy		*logind_proxy;
 	guint                    logind_sleep_id;
 	int                      logind_delay_inhibitor_fd;
 
+	UpEnumerator		*udev_enum;
+
 	/* BlueZ */
 	guint			 bluez_watch_id;
 	GDBusObjectManager	*bluez_client;
@@ -79,9 +82,6 @@ static guint signals [SIGNAL_LAST] = { 0 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (UpBackend, up_backend, G_TYPE_OBJECT)
 
-static gboolean up_backend_device_add (UpBackend *backend, GUdevDevice *native, const char *was_event);
-static void up_backend_device_remove (UpBackend *backend, GUdevDevice *native);
-
 static void
 input_switch_changed_cb (UpInput   *input,
 			 gboolean   switch_value,
@@ -90,208 +90,29 @@ input_switch_changed_cb (UpInput   *input,
 	up_daemon_set_lid_is_closed (backend->priv->daemon, switch_value);
 }
 
-static gpointer
-is_macbook (gpointer data)
-{
-	g_autofree char *product = NULL;
-
-	if (!g_file_get_contents ("/sys/devices/virtual/dmi/id/product_name", &product, NULL, NULL) ||
-	    product == NULL)
-		return GINT_TO_POINTER(FALSE);
-	return GINT_TO_POINTER(g_str_has_prefix (product, "MacBook"));
-}
-
-gboolean
-up_backend_needs_poll_after_uevent (void)
-{
-	static GOnce dmi_once = G_ONCE_INIT;
-	g_once (&dmi_once, is_macbook, NULL);
-	return GPOINTER_TO_INT(dmi_once.retval);
-}
-
-static UpDevice *
-up_backend_device_new (UpBackend *backend, GUdevDevice *native)
-{
-	const gchar *subsys;
-	const gchar *native_path;
-	UpDevice *device = NULL;
-	UpInput *input;
-	gboolean ret;
-
-	subsys = g_udev_device_get_subsystem (native);
-	if (g_strcmp0 (subsys, "power_supply") == 0) {
-
-		/* are we a valid power supply */
-		device = UP_DEVICE (up_device_supply_new ());
-		g_object_set (G_OBJECT(device),
-			      "ignore-system-percentage", GPOINTER_TO_INT (is_macbook (NULL)),
-			      NULL);
-		ret = up_device_coldplug (device, backend->priv->daemon, G_OBJECT (native));
-		if (ret)
-			goto out;
-
-		/* no valid power supply object */
-		g_clear_object (&device);
-
-	} else if (g_strcmp0 (subsys, "tty") == 0) {
-
-		/* see if this is a Watts Up Pro device */
-		device = UP_DEVICE (up_device_wup_new ());
-		ret = up_device_coldplug (device, backend->priv->daemon, G_OBJECT (native));
-		if (ret)
-			goto out;
-
-		/* no valid TTY object */
-		g_clear_object (&device);
-
-	} else if (g_strcmp0 (subsys, "usb") == 0 || g_strcmp0 (subsys, "usbmisc") == 0) {
-
-#ifdef HAVE_IDEVICE
-		/* see if this is an iDevice */
-		device = UP_DEVICE (up_device_idevice_new ());
-		ret = up_device_coldplug (device, backend->priv->daemon, G_OBJECT (native));
-		if (ret)
-			goto out;
-		g_object_unref (device);
-#endif /* HAVE_IDEVICE */
-
-		/* try to detect a HID UPS */
-		device = UP_DEVICE (up_device_hid_new ());
-		ret = up_device_coldplug (device, backend->priv->daemon, G_OBJECT (native));
-		if (ret)
-			goto out;
-
-		/* no valid USB object */
-		g_clear_object (&device);
-
-	} else if (g_strcmp0 (subsys, "input") == 0) {
-
-		/* check input device */
-		input = up_input_new ();
-		ret = up_input_coldplug (input, native);
-		if (ret) {
-			/* we now have a lid */
-			up_daemon_set_lid_is_present (backend->priv->daemon, TRUE);
-			g_signal_connect (G_OBJECT (input), "switch-changed",
-					  G_CALLBACK (input_switch_changed_cb), backend);
-			up_daemon_set_lid_is_closed (backend->priv->daemon,
-						     up_input_get_switch_value (input));
-
-			/* not a power device, add it to the managed devices
-			 * and don't return a power device */
-			up_device_list_insert (backend->priv->managed_devices, G_OBJECT (native), G_OBJECT (input));
-			device = NULL;
-		}
-		g_object_unref (input);
-	} else {
-		native_path = g_udev_device_get_sysfs_path (native);
-		g_warning ("native path %s (%s) ignoring", native_path, subsys);
-	}
-out:
-	return device;
-}
-
-static void
-up_backend_device_changed (UpBackend *backend, GUdevDevice *native, const char *was_event)
-{
-	GObject *object;
-	UpDevice *device;
-	gboolean ret;
-
-	/* first, check the device and add it if it doesn't exist */
-	object = up_device_list_lookup (backend->priv->device_list, G_OBJECT (native));
-	if (object == NULL) {
-		up_backend_device_add (backend, native, "changed");
-		goto out;
-	}
-
-	/* need to refresh device */
-	device = UP_DEVICE (object);
-	ret = up_device_refresh_internal (device);
-	if (!ret) {
-		g_debug ("no changes on %s", up_device_get_object_path (device));
-		goto out;
-	}
-
-	if (was_event)
-		g_warning ("treated %s event as change on %s", was_event, g_udev_device_get_sysfs_path (native));
-
-out:
-	g_clear_object (&object);
-}
-
-static gboolean
-up_backend_device_add (UpBackend *backend, GUdevDevice *native, const char *was_event)
-{
-	GObject *object;
-	UpDevice *device;
-	gboolean ret = TRUE;
-
-	/* does device exist in db? */
-	object = up_device_list_lookup (backend->priv->device_list, G_OBJECT (native));
-	if (object != NULL) {
-		device = UP_DEVICE (object);
-		/* we already have the device; treat as change event */
-		up_backend_device_changed (backend, native, "add");
-		goto out;
-	}
-
-	/* get the right sort of device */
-	device = up_backend_device_new (backend, native);
-	if (device == NULL) {
-		ret = FALSE;
-		goto out;
-	}
-
-	if (was_event)
-		g_warning ("treated %s event as add on %s", was_event, g_udev_device_get_sysfs_path (native));
-
-	/* emit */
-	g_signal_emit (backend, signals[SIGNAL_DEVICE_ADDED], 0, native, device);
-out:
-	g_clear_object (&object);
-	return ret;
-}
-
-static void
-up_backend_device_remove (UpBackend *backend, GUdevDevice *native)
-{
-	GObject *object;
-	UpDevice *device;
-
-	/* does device exist in db? */
-	object = up_device_list_lookup (backend->priv->device_list, G_OBJECT (native));
-	if (object == NULL) {
-		g_debug ("ignoring remove event on %s", g_udev_device_get_sysfs_path (native));
-		goto out;
-	}
-
-	device = UP_DEVICE (object);
-	/* emit */
-	g_debug ("emitting device-removed: %s", g_udev_device_get_sysfs_path (native));
-	g_signal_emit (backend, signals[SIGNAL_DEVICE_REMOVED], 0, native, device);
-
-out:
-	g_clear_object (&object);
-}
-
 static void
 up_backend_uevent_signal_handler_cb (GUdevClient *client, const gchar *action,
 				      GUdevDevice *device, gpointer user_data)
 {
 	UpBackend *backend = UP_BACKEND (user_data);
+	g_autoptr(UpInput) input = NULL;
 
-	if (g_strcmp0 (action, "add") == 0) {
-		g_debug ("SYSFS add %s", g_udev_device_get_sysfs_path (device));
-		up_backend_device_add (backend, device, NULL);
-	} else if (g_strcmp0 (action, "remove") == 0) {
-		g_debug ("SYSFS remove %s", g_udev_device_get_sysfs_path (device));
-		up_backend_device_remove (backend, device);
-	} else if (g_strcmp0 (action, "change") == 0) {
-		g_debug ("SYSFS change %s", g_udev_device_get_sysfs_path (device));
-		up_backend_device_changed (backend, device, NULL);
-	} else {
-		g_debug ("unhandled action '%s' on %s", action, g_udev_device_get_sysfs_path (device));
+	if (backend->priv->lid_device)
+		return;
+
+	if (g_strcmp0 (action, "add") != 0)
+		return;
+
+	/* check if the input device is a lid */
+	input = up_input_new ();
+	if (up_input_coldplug (input, device)) {
+		up_daemon_set_lid_is_present (backend->priv->daemon, TRUE);
+		g_signal_connect (G_OBJECT (input), "switch-changed",
+				  G_CALLBACK (input_switch_changed_cb), backend);
+		up_daemon_set_lid_is_closed (backend->priv->daemon,
+					     up_input_get_switch_value (input));
+
+		backend->priv->lid_device = g_steal_pointer (&input);
 	}
 }
 
@@ -358,7 +179,7 @@ bluez_interface_removed (GDBusObjectManager *manager,
 		return;
 
 	g_debug ("emitting device-removed: %s", g_dbus_object_get_object_path (bus_object));
-	g_signal_emit (backend, signals[SIGNAL_DEVICE_REMOVED], 0, bus_object, UP_DEVICE (object));
+	g_signal_emit (backend, signals[SIGNAL_DEVICE_REMOVED], 0, UP_DEVICE (object));
 
 	g_object_unref (object);
 }
@@ -369,10 +190,9 @@ bluez_interface_added (GDBusObjectManager *manager,
 		       GDBusInterface     *interface,
 		       gpointer            user_data)
 {
+	g_autoptr(UpDevice) device = NULL;
 	UpBackend *backend = user_data;
-	UpDevice *device;
 	GObject *object;
-	gboolean ret;
 
 	if (!has_battery_iface (bus_object))
 		return;
@@ -383,15 +203,14 @@ bluez_interface_added (GDBusObjectManager *manager,
 		return;
 	}
 
-	device = UP_DEVICE (up_device_bluez_new ());
-	ret = up_device_coldplug (device, backend->priv->daemon, G_OBJECT (bus_object));
-	if (!ret) {
-		g_object_unref (device);
-		return;
+	device = g_initable_new (UP_TYPE_DEVICE_BLUEZ, NULL, NULL,
+	                         "daemon", backend->priv->daemon,
+	                         "native", G_OBJECT (bus_object),
+	                         NULL);
+	if (device) {
+		g_debug ("emitting device-added: %s", g_dbus_object_get_object_path (bus_object));
+		g_signal_emit (backend, signals[SIGNAL_DEVICE_ADDED], 0, device);
 	}
-
-	g_debug ("emitting device-added: %s", g_dbus_object_get_object_path (bus_object));
-	g_signal_emit (backend, signals[SIGNAL_DEVICE_ADDED], 0, bus_object, device);
 }
 
 static void
@@ -470,7 +289,7 @@ bluez_vanished (GDBusConnection *connection,
 
 			object = G_DBUS_OBJECT (up_device_get_native (device));
 			g_debug ("emitting device-removed: %s", g_dbus_object_get_object_path (object));
-			g_signal_emit (backend, signals[SIGNAL_DEVICE_REMOVED], 0, object, UP_DEVICE (object));
+			g_signal_emit (backend, signals[SIGNAL_DEVICE_REMOVED], 0, device);
 		}
 	}
 
@@ -479,6 +298,20 @@ bluez_vanished (GDBusConnection *connection,
 	g_clear_object (&backend->priv->bluez_client);
 }
 
+static void
+udev_device_added_cb (UpBackend *backend, UpDevice *device)
+{
+	g_debug ("Got new device from udev enumerator: %p", device);
+	g_signal_emit (backend, signals[SIGNAL_DEVICE_ADDED], 0, device);
+}
+
+static void
+udev_device_removed_cb (UpBackend *backend, UpDevice *device)
+{
+	g_debug ("Removing device from udev enumerator: %p", device);
+	g_signal_emit (backend, signals[SIGNAL_DEVICE_REMOVED], 0, device);
+}
+
 /**
  * up_backend_coldplug:
  * @backend: The %UpBackend class instance
@@ -492,32 +325,24 @@ bluez_vanished (GDBusConnection *connection,
 gboolean
 up_backend_coldplug (UpBackend *backend, UpDaemon *daemon)
 {
-	GUdevDevice *native;
-	GList *devices;
+	g_autolist(GUdevDevice) devices = NULL;
 	GList *l;
-	guint i;
-	const gchar *subsystems_wup[] = {"power_supply", "usb", "usbmisc", "tty", "input", NULL};
-	const gchar *subsystems[] = {"power_supply", "usb", "usbmisc", "input", NULL};
 
 	backend->priv->daemon = g_object_ref (daemon);
 	backend->priv->device_list = up_daemon_get_device_list (daemon);
-	if (up_config_get_boolean (backend->priv->config, "EnableWattsUpPro"))
-		backend->priv->gudev_client = g_udev_client_new (subsystems_wup);
-	else
-		backend->priv->gudev_client = g_udev_client_new (subsystems);
+
+	/* Watch udev for input devices to find the lid switch */
+	backend->priv->gudev_client = g_udev_client_new ((const char *[]){ "input", NULL });
 	g_signal_connect (backend->priv->gudev_client, "uevent",
 			  G_CALLBACK (up_backend_uevent_signal_handler_cb), backend);
 
 	/* add all subsystems */
-	for (i=0; subsystems[i] != NULL; i++) {
-		g_debug ("registering subsystem : %s", subsystems[i]);
-		devices = g_udev_client_query_by_subsystem (backend->priv->gudev_client, subsystems[i]);
-		for (l = devices; l != NULL; l = l->next) {
-			native = l->data;
-			up_backend_device_add (backend, native, NULL);
-		}
-		g_list_free_full (devices, (GDestroyNotify) g_object_unref);
-	}
+	devices = g_udev_client_query_by_subsystem (backend->priv->gudev_client, "input");
+	for (l = devices; l != NULL; l = l->next)
+		up_backend_uevent_signal_handler_cb (backend->priv->gudev_client,
+						     "add",
+						     G_UDEV_DEVICE (l->data),
+						     backend);
 
 	backend->priv->bluez_watch_id = g_bus_watch_name (G_BUS_TYPE_SYSTEM,
 							  "org.bluez",
@@ -527,6 +352,17 @@ up_backend_coldplug (UpBackend *backend, UpDaemon *daemon)
 							  backend,
 							  NULL);
 
+	backend->priv->udev_enum = g_object_new (UP_TYPE_ENUMERATOR_UDEV,
+						 "daemon", daemon,
+						 NULL);
+
+	g_signal_connect_swapped (backend->priv->udev_enum, "device-added",
+				  G_CALLBACK (udev_device_added_cb), backend);
+	g_signal_connect_swapped (backend->priv->udev_enum, "device-removed",
+				  G_CALLBACK (udev_device_removed_cb), backend);
+
+	g_assert (g_initable_init (G_INITABLE (backend->priv->udev_enum), NULL, NULL));
+
 	return TRUE;
 }
 
@@ -541,10 +377,9 @@ void
 up_backend_unplug (UpBackend *backend)
 {
 	g_clear_object (&backend->priv->gudev_client);
+	g_clear_object (&backend->priv->udev_enum);
 	g_clear_object (&backend->priv->device_list);
-	/* set in init, clear the list to remove reference to UpDaemon */
-	if (backend->priv->managed_devices != NULL)
-		up_device_list_clear (backend->priv->managed_devices, FALSE);
+	g_clear_object (&backend->priv->lid_device);
 	g_clear_object (&backend->priv->daemon);
 	if (backend->priv->bluez_watch_id > 0) {
 		g_bus_unwatch_name (backend->priv->bluez_watch_id);
@@ -762,7 +597,7 @@ up_backend_prepare_for_sleep (GDBusConnection *connection,
 
 	for (i = 0; i < array->len; i++) {
 		UpDevice *device = UP_DEVICE (g_ptr_array_index (array, i));
-		up_device_refresh_internal (device);
+		up_device_refresh_internal (device, UP_REFRESH_RESUME);
 	}
 
 	g_ptr_array_unref (array);
@@ -782,13 +617,13 @@ up_backend_class_init (UpBackendClass *klass)
 			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (UpBackendClass, device_added),
 			      NULL, NULL, NULL,
-			      G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_POINTER);
+			      G_TYPE_NONE, 1, UP_TYPE_DEVICE);
 	signals [SIGNAL_DEVICE_REMOVED] =
 		g_signal_new ("device-removed",
 			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (UpBackendClass, device_removed),
 			      NULL, NULL, NULL,
-			      G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_POINTER);
+			      G_TYPE_NONE, 1, UP_TYPE_DEVICE);
 }
 
 static void
@@ -799,7 +634,6 @@ up_backend_init (UpBackend *backend)
 
 	backend->priv = up_backend_get_instance_private (backend);
 	backend->priv->config = up_config_new ();
-	backend->priv->managed_devices = up_device_list_new ();
 	backend->priv->logind_proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
 								     0,
 								     NULL,
@@ -856,7 +690,7 @@ up_backend_finalize (GObject *object)
 
 	g_clear_object (&backend->priv->logind_proxy);
 
-	g_object_unref (backend->priv->managed_devices);
+	g_clear_object (&backend->priv->lid_device);
 
 	G_OBJECT_CLASS (up_backend_parent_class)->finalize (object);
 }
