@@ -23,26 +23,23 @@
 
 #include "up-backend.h"
 #include "up-daemon.h"
-#include "up-marshal.h"
 #include "up-device.h"
+#include "up-config.h"
+#include "up-backend-bsd-private.h"
 #include <string.h> /* strcmp() */
 
-#define UP_BACKEND_SUSPEND_COMMAND	"/usr/sbin/zzz"
-
 static void	up_backend_class_init	(UpBackendClass	*klass);
 static void	up_backend_init	(UpBackend		*backend);
 static void	up_backend_finalize	(GObject		*object);
 
-static gboolean	up_backend_apm_get_power_info(int, struct apm_power_info*);
+static gboolean	up_backend_apm_get_power_info(struct apm_power_info*);
 UpDeviceState up_backend_apm_get_battery_state_value(u_char battery_state);
 static void	up_backend_update_acpibat_state(UpDevice*, struct sensordev);
+static void	up_backend_update_lid_status(UpDaemon*);
 
 static gboolean		up_apm_device_get_on_battery	(UpDevice *device, gboolean *on_battery);
-static gboolean		up_apm_device_get_low_battery	(UpDevice *device, gboolean *low_battery);
 static gboolean		up_apm_device_get_online		(UpDevice *device, gboolean *online);
-static gboolean		up_apm_device_refresh		(UpDevice *device);
-
-#define UP_BACKEND_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), UP_TYPE_BACKEND, UpBackendPrivate))
+static gboolean		up_apm_device_refresh		(UpDevice *device, UpRefreshReason reason);
 
 struct UpBackendPrivate
 {
@@ -51,7 +48,8 @@ struct UpBackendPrivate
 	UpDevice		*battery;
 	GThread			*apm_thread;
 	gboolean		is_laptop;
-	int			apm_fd;
+	UpConfig		*config;
+	GDBusProxy		*seat_manager_proxy;
 };
 
 enum {
@@ -62,9 +60,7 @@ enum {
 
 static guint signals [SIGNAL_LAST] = { 0 };
 
-int apm_fd; /* ugly global.. needs to move to a device native object */
-
-G_DEFINE_TYPE (UpBackend, up_backend, G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE (UpBackend, up_backend, G_TYPE_OBJECT)
 
 /**
  * functions called by upower daemon
@@ -97,28 +93,6 @@ up_apm_device_get_on_battery (UpDevice *device, gboolean * on_battery)
 	*on_battery = (state == UP_DEVICE_STATE_DISCHARGING);
 	return TRUE;
 }
-gboolean
-up_apm_device_get_low_battery (UpDevice *device, gboolean * low_battery)
-{
-	gboolean ret;
-	gboolean on_battery;
-	gdouble percentage;
-
-	g_return_val_if_fail (low_battery != NULL, FALSE);
-
-	ret = up_apm_device_get_on_battery (device, &on_battery);
-	if (!ret)
-		return FALSE;
-
-	if (!on_battery) {
-		*low_battery = FALSE;
-		return TRUE;
-	}
-
-	g_object_get (device, "percentage", &percentage, (void*) NULL);
-	*low_battery = (percentage < 10.0f);
-	return TRUE;
-}
 
 gboolean
 up_apm_device_get_online (UpDevice *device, gboolean * online)
@@ -153,105 +127,87 @@ up_apm_device_get_online (UpDevice *device, gboolean * online)
 gboolean
 up_backend_coldplug (UpBackend *backend, UpDaemon *daemon)
 {
-	UpApmNative *acnative = NULL;
-	UpApmNative *battnative = NULL;
 	backend->priv->daemon = g_object_ref (daemon);
-	/* XXX no way to get lid status atm */
-	up_daemon_set_lid_is_present (backend->priv->daemon, FALSE);
+
 	if (backend->priv->is_laptop)
 	{
-		acnative = up_apm_native_new("/ac");
-		if (!up_device_coldplug (backend->priv->ac, backend->priv->daemon, G_OBJECT(acnative)))
+		up_backend_update_lid_status(daemon);
+		if (!g_initable_init (G_INITABLE (backend->priv->ac), NULL, NULL))
 			g_warning ("failed to coldplug ac");
 		else
-			g_signal_emit (backend, signals[SIGNAL_DEVICE_ADDED], 0, acnative, backend->priv->ac);
+			g_signal_emit (backend, signals[SIGNAL_DEVICE_ADDED], 0, backend->priv->ac);
 
-		battnative = up_apm_native_new("/batt");
-		if (!up_device_coldplug (backend->priv->battery, backend->priv->daemon, G_OBJECT(battnative)))
+		if (!g_initable_init (G_INITABLE (backend->priv->battery), NULL, NULL))
 			g_warning ("failed to coldplug battery");
 		else
-			g_signal_emit (backend, signals[SIGNAL_DEVICE_ADDED], 0, battnative, backend->priv->battery);
+			g_signal_emit (backend, signals[SIGNAL_DEVICE_ADDED], 0, backend->priv->battery);
 	}
 
 	return TRUE;
 }
 
-
 /**
- * up_backend_get_powersave_command:
- **/
-const gchar *
-up_backend_get_powersave_command (UpBackend *backend, gboolean powersave)
+ * up_backend_unplug:
+ * @backend: The %UpBackend class instance
+ *
+ * Forget about all learned devices, effectively undoing up_backend_coldplug.
+ * Resources are released without emitting signals.
+ */
+void
+up_backend_unplug (UpBackend *backend)
 {
-	return NULL;
+	if (backend->priv->daemon != NULL) {
+		g_object_unref (backend->priv->daemon);
+		backend->priv->daemon = NULL;
+	}
 }
 
 /**
- * up_backend_get_suspend_command:
- **/
-const gchar *
-up_backend_get_suspend_command (UpBackend *backend)
+ * up_backend_get_seat_manager_proxy:
+ * @backend: The %UpBackend class instance
+ *
+ * Returns the seat manager object or NULL on error. [transfer none]
+ */
+GDBusProxy *
+up_backend_get_seat_manager_proxy (UpBackend  *backend)
 {
-	return UP_BACKEND_SUSPEND_COMMAND;
+	g_return_val_if_fail (UP_IS_BACKEND (backend), NULL);
+
+	return backend->priv->seat_manager_proxy;
 }
 
 /**
- * up_backend_get_hibernate_command:
- **/
-const gchar *
-up_backend_get_hibernate_command (UpBackend *backend)
+ * up_backend_get_config:
+ * @backend: The %UpBackend class instance
+ *
+ * Returns the UpConfig object or NULL on error. [transfer none]
+ */
+UpConfig *
+up_backend_get_config (UpBackend  *backend)
 {
-	return NULL;
+	g_return_val_if_fail (UP_IS_BACKEND (backend), NULL);
+
+	return backend->priv->config;
 }
 
-/**
- * up_backend_kernel_can_suspend:
- **/
-gboolean
-up_backend_kernel_can_suspend (UpBackend *backend)
-{
-	return TRUE;
-}
-
-/**
- * up_backend_kernel_can_hibernate:
- **/
-gboolean
-up_backend_kernel_can_hibernate (UpBackend *backend)
-{
-	return FALSE;
-}
-
-gboolean
-up_backend_has_encrypted_swap (UpBackend *backend)
-{
-	return FALSE;
-}
-
-/* Return value: a percentage value */
-gfloat
-up_backend_get_used_swap (UpBackend *backend)
-{
-	return 0;
-}
 
 /**
  * OpenBSD specific code
  **/
 
 static gboolean
-up_backend_apm_get_power_info(int fd, struct apm_power_info *bstate) {
+up_backend_apm_get_power_info(struct apm_power_info *bstate) {
 	bstate->battery_state = 255;
 	bstate->ac_state = 255;
 	bstate->battery_life = 0;
 	bstate->minutes_left = -1;
 
-	if (fd == 0)
-		return TRUE; /* cheat, defaulting values */
-	if (-1 == ioctl(fd, APM_IOC_GETPOWER, bstate)) {
-		g_warning("ioctl on fd %d failed : %s", fd, g_strerror(errno));
+#ifndef UPOWER_CI_DISABLE_PLATFORM_CODE
+	if (-1 == ioctl(up_apm_get_fd(), APM_IOC_GETPOWER, bstate)) {
+		g_error("ioctl on apm fd failed : %s", g_strerror(errno));
 		return FALSE;
 	}
+#endif
 	return TRUE;
 }
 
@@ -279,7 +235,8 @@ up_backend_update_ac_state(UpDevice* device)
 	gboolean ret, new_is_online, cur_is_online;
 	struct apm_power_info a;
 
-	ret = up_backend_apm_get_power_info(apm_fd, &a);
+	up_backend_update_lid_status(up_device_get_daemon(device));
+	ret = up_backend_apm_get_power_info(&a);
 	if (!ret)
 		return ret;
 
@@ -300,13 +257,13 @@ static gboolean
 up_backend_update_battery_state(UpDevice* device)
 {
 	gdouble percentage;
-	gboolean ret;
+	gboolean ret, is_present;
 	struct sensordev sdev;
 	UpDeviceState cur_state, new_state;
 	gint64 cur_time_to_empty, new_time_to_empty;
 	struct apm_power_info a;
 
-	ret = up_backend_apm_get_power_info(apm_fd, &a);
+	ret = up_backend_apm_get_power_info(&a);
 	if (!ret)
 		return ret;
 
@@ -314,18 +271,52 @@ up_backend_update_battery_state(UpDevice* device)
 		"state", &cur_state,
 		"percentage", &percentage,
 		"time-to-empty", &cur_time_to_empty,
+		"is-present", &is_present,
 		(void*) NULL);
 
+	// zero out new_time_to empty if we're not discharging or minutes_left is negative
+	new_time_to_empty = (new_state == UP_DEVICE_STATE_DISCHARGING && a.minutes_left > 0 ? a.minutes_left : 0);
+
 	/* XXX use acpibat0.raw0 if available */
+	/*
+	 * XXX: Stop having a split brain regarding
+	 * up_backend_apm_get_battery_state_value(). Either move the state
+	 * setting code below into that function, or inline that function here.
+	 */
 	new_state = up_backend_apm_get_battery_state_value(a.battery_state);
 	// if percentage/minutes goes down or ac is off, we're likely discharging..
 	if (percentage < a.battery_life || cur_time_to_empty < new_time_to_empty || a.ac_state == APM_AC_OFF)
 		new_state = UP_DEVICE_STATE_DISCHARGING;
-	if (a.ac_state == APM_AC_ON)
-		new_state = UP_DEVICE_STATE_CHARGING;
+	/*
+	 * If we're on AC, we may either be charging, or the battery is already
+	 * fully charged. Figure out which.
+	 */
+	if (a.ac_state == APM_AC_ON) {
+		if ((gdouble) a.battery_life >= 99.0)
+			new_state = UP_DEVICE_STATE_FULLY_CHARGED;
+		else
+			new_state = UP_DEVICE_STATE_CHARGING;
+	}
 
-	// zero out new_time_to empty if we're not discharging
-	new_time_to_empty = (new_state == UP_DEVICE_STATE_DISCHARGING ? a.minutes_left : 0);
+	if ((a.battery_state == APM_BATTERY_ABSENT) ||
+	    (a.battery_state == APM_BATT_UNKNOWN)) {
+		/* Reset some known fields which remain untouched below. */
+		g_object_set(device,
+			     "is-rechargeable", FALSE,
+			     "energy", (gdouble) 0.0,
+			     "energy-empty", (gdouble) 0.0,
+			     "energy-full", (gdouble) 0.0,
+			     "energy-full-design", (gdouble) 0.0,
+			     "energy-rate", (gdouble) 0.0,
+			     NULL);
+		is_present = FALSE;
+		if (a.battery_state == APM_BATTERY_ABSENT)
+			new_state = UP_DEVICE_STATE_EMPTY;
+		else
+			new_state = UP_DEVICE_STATE_UNKNOWN;
+	} else {
+		is_present = TRUE;
+	}
 
 	if (cur_state != new_state ||
 		percentage != (gdouble) a.battery_life ||
@@ -335,6 +326,7 @@ up_backend_update_battery_state(UpDevice* device)
 			"state", new_state,
 			"percentage", (gdouble) a.battery_life,
 			"time-to-empty", new_time_to_empty * 60,
+			"is-present", is_present,
 			(void*) NULL);
 		if(up_native_get_sensordev("acpibat0", &sdev))
 			up_backend_update_acpibat_state(device, sdev);
@@ -347,9 +339,11 @@ up_backend_update_battery_state(UpDevice* device)
 static void
 up_backend_update_acpibat_state(UpDevice* device, struct sensordev s)
 {
-	enum sensor_type type, typev = SENSOR_INTEGER;
+#ifndef UPOWER_CI_DISABLE_PLATFORM_CODE
+	enum sensor_type type;
 	int numt;
-	gdouble bst_volt, bif_dvolt, bst_rate, bif_lastfullcap, bst_cap, bif_dcap, bif_lowcap, capacity;
+	gdouble bst_volt, bst_rate, bif_cap, bif_lastfullcap, bst_cap, bif_lowcap, capacity;
+	/* gdouble bif_dvolt; */
 	struct sensor sens;
 	size_t slen = sizeof(sens);
 	int mib[] = {CTL_HW, HW_SENSORS, 0, 0, 0};
@@ -364,42 +358,49 @@ up_backend_update_acpibat_state(UpDevice* device, struct sensordev s)
 			else if (slen > 0 && (sens.flags & SENSOR_FINVALID) == 0) {
 				if (sens.type == SENSOR_VOLTS_DC && !strcmp(sens.desc, "current voltage"))
 					bst_volt = sens.value / 1000000.0f;
+				if ((sens.type == SENSOR_AMPHOUR || sens.type == SENSOR_WATTHOUR) && !strcmp(sens.desc, "design capacity")) {
+					bif_cap = (sens.type == SENSOR_AMPHOUR ? bst_volt : 1) * sens.value / 1000000.0f;
+				}
 				if ((sens.type == SENSOR_AMPHOUR || sens.type == SENSOR_WATTHOUR) && !strcmp(sens.desc, "last full capacity")) {
-					typev = sens.type;
-					bif_lastfullcap = sens.value / 1000000.0f;
+					bif_lastfullcap = (sens.type == SENSOR_AMPHOUR ? bst_volt : 1) * sens.value / 1000000.0f;
 				}
 				if ((sens.type == SENSOR_AMPHOUR || sens.type == SENSOR_WATTHOUR) && !strcmp(sens.desc, "low capacity")) {
-					typev = sens.type;
-					bif_lowcap = sens.value / 1000000.0f;
+					bif_lowcap = (sens.type == SENSOR_AMPHOUR ? bst_volt : 1) * sens.value / 1000000.0f;
 				}
 				if ((sens.type == SENSOR_AMPHOUR || sens.type == SENSOR_WATTHOUR) && !strcmp(sens.desc, "remaining capacity")) {
-					typev = sens.type;
-					bst_cap = sens.value / 1000000.0f;
+					bst_cap = (sens.type == SENSOR_AMPHOUR ? bst_volt : 1) * sens.value / 1000000.0f;
+				}
+				if ((sens.type == SENSOR_AMPS || sens.type == SENSOR_WATTS) && !strcmp(sens.desc, "rate")) {
+					bst_rate = (sens.type == SENSOR_AMPS ? bst_volt : 1) * sens.value / 1000000.0f;
 				}
-				if (sens.type == SENSOR_INTEGER && !strcmp(sens.desc, "rate"))
-					bst_rate = sens.value / 1000.0f;
 				/*
 				bif_dvolt = "voltage" = unused ?
-				capacity = lastfull/dcap * 100 ?
 				amphour1 = warning capacity ?
 				raw0 = battery state
 				*/
 			}
 		}
 	}
-	if (typev == SENSOR_AMPHOUR) {
-		bst_cap *= bst_volt;
-		bif_lowcap *= bst_volt;
-		bif_lastfullcap *= bst_volt;
-		bst_rate *= bst_volt;
+
+	capacity = 0.0f;
+	if(bif_lastfullcap > 0 && bif_cap > 0) {
+		capacity = (bif_lastfullcap / bif_cap) * 100.0f;
+		if (capacity < 0)
+			capacity = 0.0f;
+		if (capacity > 100.0)
+			capacity = 100.0f;
 	}
+
 	g_object_set (device,
 		"energy", bst_cap,
 		"energy-full", bif_lastfullcap,
+		"energy-full-design", bif_cap,
 		"energy-rate", bst_rate,
 		"energy-empty", bif_lowcap,
 		"voltage", bst_volt,
+		"capacity", capacity,
 		(void*) NULL);
+#endif
 }
 
 /* callback updating the device */
@@ -407,23 +408,20 @@ static gboolean
 up_backend_apm_powerchange_event_cb(gpointer object)
 {
 	UpBackend *backend;
-	struct apm_power_info a;
 
-	g_return_if_fail (UP_IS_BACKEND (object));
+	g_return_val_if_fail (UP_IS_BACKEND (object), FALSE);
 	backend = UP_BACKEND (object);
-	up_apm_device_refresh(backend->priv->ac);
-	up_apm_device_refresh(backend->priv->battery);
+	up_device_refresh_internal (backend->priv->ac, UP_REFRESH_EVENT);
+	up_device_refresh_internal (backend->priv->battery, UP_REFRESH_EVENT);
 	/* return false to not endless loop */
 	return FALSE;
 }
 
 static gboolean
-up_apm_device_refresh(UpDevice* device)
+up_apm_device_refresh(UpDevice* device, UpRefreshReason reason)
 {
 	UpDeviceKind type;
-	GTimeVal timeval;
 	gboolean ret;
-
 	g_object_get (device, "type", &type, NULL);
 
 	switch (type) {
@@ -438,38 +436,97 @@ up_apm_device_refresh(UpDevice* device)
 			break;
 	}
 
-	if (ret) {
-		g_get_current_time (&timeval);
-		g_object_set (device, "update-time", (guint64) timeval.tv_sec, NULL);
-	}
+	if (ret)
+		g_object_set (device, "update-time", (guint64) g_get_real_time () / G_USEC_PER_SEC, NULL);
 
 	return ret;
 }
 
+/*
+ * Check the lid status, return TRUE if one was found, FALSE otherwise.
+ */
+static void
+up_backend_update_lid_status(UpDaemon *daemon) {
+#ifndef UPOWER_CI_DISABLE_PLATFORM_CODE
+	/* Use hw.sensors.acpibtn0.indicator0=On (lid open) */
+	struct sensordev sensordev;
+	struct sensor sensor;
+	size_t sdlen, slen;
+	int dev, numt, mib[5] = {CTL_HW, HW_SENSORS, 0, 0, 0};
+	gboolean lid_found = FALSE;
+	gboolean lid_open = FALSE;
+
+	sdlen = sizeof(struct sensordev);
+	slen  = sizeof(struct sensor);
+
+	/* go through all acpibtn devices, and check if one of the values match "lid"
+	   if so, use that device.
+	*/
+	for (dev = 0; ; dev++) {
+		mib[2] = dev;
+		if (sysctl(mib, 3, &sensordev, &sdlen, NULL, 0) == -1) {
+			if (errno == ENXIO)
+				continue;
+			if (errno == ENOENT)
+				break;
+		}
+
+		if (strstr(sensordev.xname, "acpibtn") != NULL) {
+			mib[3] = SENSOR_INDICATOR;
+			for (numt = 0; numt < sensordev.maxnumt[SENSOR_INDICATOR]; numt++) {
+				mib[4] = numt;
+				if (sysctl(mib, 5, &sensor, &slen, NULL, 0) == -1) {
+					if (errno != ENOENT) {
+						g_warning("failed to get sensor data from %s",
+							  sensordev.xname);
+						continue;
+					}
+				}
+
+				/*
+				 * Found an acpibtn device, now check if the
+				 * description has got anything with a lid in it.
+				 */
+				if (strstr(sensor.desc, "lid open") == NULL) {
+					g_warning ("nothing here for %s with %s\n",
+						   sensordev.xname, sensor.desc);
+					continue;
+				} else {
+					lid_found = TRUE;
+					if (sensor.value)
+						lid_open = TRUE;
+					else
+						lid_open = FALSE;
+				}
+			}
+		}
+	}
+
+	up_daemon_set_lid_is_present (daemon, lid_found);
+	up_daemon_set_lid_is_closed (daemon, !lid_open);
+#endif
+}
+
 /* thread doing kqueue() on apm device */
 static gpointer
 up_backend_apm_event_thread(gpointer object)
 {
+#ifndef UPOWER_CI_DISABLE_PLATFORM_CODE
 	int kq, nevents;
 	struct kevent ev;
 	struct timespec ts = {600, 0}, sts = {0, 0};
 
 	UpBackend *backend;
 
-	g_return_if_fail (UP_IS_BACKEND (object));
+	g_return_val_if_fail (UP_IS_BACKEND (object), NULL);
 	backend = UP_BACKEND (object);
 
 	g_debug("setting up apm thread");
 
-	/* open /dev/apm */
-	if ((apm_fd = open("/dev/apm", O_RDONLY)) == -1) {
-		if (errno != ENXIO && errno != ENOENT)
-			g_error("cannot open device file");
-	}
 	kq = kqueue();
 	if (kq <= 0)
 		g_error("kqueue");
-	EV_SET(&ev, apm_fd, EVFILT_READ, EV_ADD | EV_ENABLE | EV_CLEAR,
+	EV_SET(&ev, up_apm_get_fd(), EVFILT_READ, EV_ADD | EV_ENABLE | EV_CLEAR,
 	    0, 0, NULL);
 	nevents = 1;
 	if (kevent(kq, &ev, nevents, NULL, 0, &sts) < 0)
@@ -485,11 +542,12 @@ up_backend_apm_event_thread(gpointer object)
 			break;
 		if (!rv)
 			continue;
-		if (ev.ident == (guint) apm_fd && APM_EVENT_TYPE(ev.data) == APM_POWER_CHANGE ) {
+		if (ev.ident == (guint) up_apm_get_fd() && APM_EVENT_TYPE(ev.data) == APM_POWER_CHANGE ) {
 			/* g_idle_add the callback */
 			g_idle_add((GSourceFunc) up_backend_apm_powerchange_event_cb, backend);
 		}
 	}
+#endif
 	return NULL;
 	/* shouldnt be reached ? */
 }
@@ -506,9 +564,7 @@ up_backend_apm_event_thread(gpointer object)
 UpBackend *
 up_backend_new (void)
 {
-	UpBackend *backend;
-	backend = g_object_new (UP_TYPE_BACKEND, NULL);
-	return UP_BACKEND (backend);
+	return g_object_new (UP_TYPE_BACKEND, NULL);
 }
 
 /**
@@ -525,16 +581,14 @@ up_backend_class_init (UpBackendClass *klass)
 		g_signal_new ("device-added",
 			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (UpBackendClass, device_added),
-			      NULL, NULL, up_marshal_VOID__POINTER_POINTER,
-			      G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_POINTER);
+			      NULL, NULL, NULL,
+			      G_TYPE_NONE, 1, UP_TYPE_DEVICE);
 	signals [SIGNAL_DEVICE_REMOVED] =
 		g_signal_new ("device-removed",
 			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (UpBackendClass, device_removed),
-			      NULL, NULL, up_marshal_VOID__POINTER_POINTER,
-			      G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_POINTER);
-
-	g_type_class_add_private (klass, sizeof (UpBackendPrivate));
+			      NULL, NULL, NULL,
+			      G_TYPE_NONE, 1, UP_TYPE_DEVICE);
 }
 
 /**
@@ -545,34 +599,42 @@ up_backend_init (UpBackend *backend)
 {
 	GError *err = NULL;
 	UpDeviceClass *device_class;
+	gint64 current_time;
 
-	backend->priv = UP_BACKEND_GET_PRIVATE (backend);
-	backend->priv->daemon = NULL;
+	backend->priv = up_backend_get_instance_private (backend);
 	backend->priv->is_laptop = up_native_is_laptop();
 	g_debug("is_laptop:%d",backend->priv->is_laptop);
 	if (backend->priv->is_laptop)
 	{
-		backend->priv->ac = UP_DEVICE(up_device_new());
-		backend->priv->battery = UP_DEVICE(up_device_new ());
+		UpApmNative *acnative = NULL;
+		UpApmNative *battnative = NULL;
+
+		acnative = up_apm_native_new("/ac");
+		battnative = up_apm_native_new("/batt");
+
+		backend->priv->ac = UP_DEVICE(up_device_new (backend->priv->daemon, G_OBJECT(acnative)));
+		backend->priv->battery = UP_DEVICE(up_device_new (backend->priv->daemon, G_OBJECT(battnative)));
+
+		g_object_unref (acnative);
+		g_object_unref (battnative);
+
 		device_class = UP_DEVICE_GET_CLASS (backend->priv->battery);
 		device_class->get_on_battery = up_apm_device_get_on_battery;
-		device_class->get_low_battery = up_apm_device_get_low_battery;
 		device_class->get_online = up_apm_device_get_online;
 		device_class->refresh = up_apm_device_refresh;
 		device_class = UP_DEVICE_GET_CLASS (backend->priv->ac);
 		device_class->get_on_battery = up_apm_device_get_on_battery;
-		device_class->get_low_battery = up_apm_device_get_low_battery;
 		device_class->get_online = up_apm_device_get_online;
 		device_class->refresh = up_apm_device_refresh;
-		g_thread_init (NULL);
 		/* creates thread */
-		if((backend->priv->apm_thread = (GThread*) g_thread_create((GThreadFunc)up_backend_apm_event_thread, backend, FALSE, &err) == NULL))
+		if((backend->priv->apm_thread = (GThread*) g_thread_try_new("apm-poller",(GThreadFunc)up_backend_apm_event_thread, (void*) backend, &err)) == NULL)
 		{
 			g_warning("Thread create failed: %s", err->message);
 			g_error_free (err);
 		}
 
 		/* setup dummy */
+		current_time = g_get_real_time () / G_USEC_PER_SEC;
 		g_object_set (backend->priv->battery,
 			      "type", UP_DEVICE_KIND_BATTERY,
 			      "power-supply", TRUE,
@@ -582,16 +644,25 @@ up_backend_init (UpBackend *backend)
 			      "state", UP_DEVICE_STATE_UNKNOWN,
 			      "percentage", 0.0f,
 			      "time-to-empty", (gint64) 0,
+			      "update-time", (guint64) current_time,
 			      (void*) NULL);
 		g_object_set (backend->priv->ac,
 			      "type", UP_DEVICE_KIND_LINE_POWER,
 			      "online", TRUE,
 			      "power-supply", TRUE,
+			      "update-time", (guint64) current_time,
 			      (void*) NULL);
-	} else {
-		backend->priv->ac = NULL;
-		backend->priv->battery = NULL;
 	}
+
+	backend->priv->config = up_config_new ();
+	backend->priv->seat_manager_proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
+									   0,
+									   NULL,
+									   CONSOLEKIT2_DBUS_NAME,
+									   CONSOLEKIT2_DBUS_PATH,
+									   CONSOLEKIT2_DBUS_INTERFACE,
+									   NULL,
+									   NULL);
 }
 /**
  * up_backend_finalize:
@@ -605,12 +676,14 @@ up_backend_finalize (GObject *object)
 
 	backend = UP_BACKEND (object);
 
+	g_object_unref (backend->priv->config);
 	if (backend->priv->daemon != NULL)
 		g_object_unref (backend->priv->daemon);
 	if (backend->priv->battery != NULL)
 		g_object_unref (backend->priv->battery);
 	if (backend->priv->ac != NULL)
 		g_object_unref (backend->priv->ac);
+	g_clear_object (&backend->priv->seat_manager_proxy);
 	/* XXX stop apm_thread ? */
 
 	G_OBJECT_CLASS (up_backend_parent_class)->finalize (object);
